const pathToNodeModulesFolder = '${appNodeModulesFolderFullPath}';

const fs = require('fs');
const path = require('path');
const babel = require(path.join(pathToNodeModulesFolder, '@babel/core'));
const sourcemap = require(path.join(pathToNodeModulesFolder, 'source-map'));
const convertsourcemap = require(path.join(pathToNodeModulesFolder, 'convert-source-map'));

const originFilePath = ${originFileFullPathQString};
const outputPath = '${outputFullPath}';
const outputPathComps = path.parse(outputPath);

const inputFilePaths = [${inputFilePathsArrayContent}];

process.on('uncaughtException', function (err) {
  process.stderr.write(err.message);
  process.exit(1);
});

var options = 
{
    caller: {name: 'CodeKit3'},
    root: '${projectFullPath}',
    exclude: 'node_modules/**',
    inputSourceMap: true,
    sourceMaps: ${sourcemapsBool},
    configFile: false,
    babelrc: ${babelrcBool},
    plugins: [${pluginsArrayContent}],
    presets: [${presetsArrayContent}],
    generatorOpts: {
        comments: ${commentsBool},
        retainLines: ${retainLinesBool},
        auxiliaryCommentBefore: ${auxCommentBeforeQString},
        auxiliaryCommentAfter: ${auxCommentAfterQString},
        moduleID: ${moduleIDQString},
        moduleIDs: ${moduleIDsBool},
    }
}

var nukeAllOptions = ${nukeAllOptionsBool};
if (nukeAllOptions === true) {
    options = {};
}


async function doIt()
{
    const map = new (sourcemap.SourceMapGenerator)({
                                                   file: outputPathComps.base,
                                                   sourceRoot: options.sourceRoot
                                                   });
    var finalCode = '';
    var offset = 0;
    
    for (let i=0, size=inputFilePaths.length; i<size; i++)
    {
        options.filename = path.parse(inputFilePaths[i]).base;
        options.sourceFileName = path.relative(outputPathComps.dir, inputFilePaths[i]);     // For source map sources array
        
        var fResult = babel.transformFileSync(inputFilePaths[i], options);
        if (!fResult) continue;
        
        finalCode += fResult.code + '\n';
        if (fResult.map)
        {
            const consumer = await new (sourcemap.SourceMapConsumer)(fResult.map);
            const sources = new Set();
            
            consumer.eachMapping(function (mapping)
                                 {
                                 if (mapping.source != null) {
                                 sources.add(mapping.source);
                                 }
                                 map.addMapping({
                                                generated: {
                                                line: mapping.generatedLine + offset,
                                                column: mapping.generatedColumn
                                                },
                                                source: mapping.source,
                                                original: mapping.source == null ? null : {
                                                line: mapping.originalLine,
                                                column: mapping.originalColumn
                                                }
                                                });
                                 });
            
            sources.forEach(source => {
                            const content = consumer.sourceContentFor(source, true);
                            
                            if (content !== null) {
                            map.setSourceContent(source, content);
                            }
                            });
            
            offset = finalCode.split('\n').length - 1;
        }
    }
    
    var finalMap = undefined;
    if (options.sourceMaps === true || options.sourceMaps === 'inline')
    {
        if (originFilePath !== null && options.inputSourceMap && fs.existsSync(outputPath + '.map') === true)
        {
            const originalMapContent = fs.readFileSync(outputPath + '.map', 'utf8');
            const originalConsumer = new (sourcemap.SourceMapConsumer)(originalMapContent);
            const currentConsumer = new (sourcemap.SourceMapConsumer)(JSON.stringify(map));
            finalMap = new (sourcemap.SourceMapGenerator)({
                                                          file: outputPathComps.base,
                                                          sourceRoot: options.sourceRoot
                                                          });
            const originSourceFileName = path.relative(outputPathComps.dir, originFilePath);
            
            currentConsumer.eachMapping(
                                        ({
                                         source,
                                         generatedLine,
                                         generatedColumn,
                                         originalLine,
                                         originalColumn,
                                         name,
                                         }) => {
                                        if (originalLine)
                                        {
                                        const original = originalConsumer.originalPositionFor({
                                                                                              line: originalLine,
                                                                                              column: originalColumn,
                                                                                              });
                                        if (original.line)
                                        {
                                        finalMap.addMapping({
                                                            generated: {
                                                            line: generatedLine,
                                                            column: generatedColumn,
                                                            },
                                                            original: {
                                                            line: original.line,
                                                            column: original.column,
                                                            },
                                                            source: originSourceFileName,
                                                            name,
                                                            });
                                        }
                                        }
                                        });
            
            const origSources = new Set();
            originalConsumer.eachMapping(
                                         ({
                                          source
                                          }) => {
                                         if (source != null) {
                                         origSources.add(source);
                                         }
                                         });
            
            origSources.forEach(source => {
                                const content = originalConsumer.sourceContentFor(source, true);
                                
                                if (content !== null) {
                                finalMap.setSourceContent(source, content);
                                }
                                });
        }
        else
        {
            finalMap = map;
        }
    }
    
    
    if (options.sourceMaps === 'inline') {
        finalCode += "\n" + convertsourcemap.fromObject(finalMap).toComment();
    }
    
    
    try {
        fs.writeFileSync(outputPath, (finalCode + '\n//# sourceMappingURL=' + outputPathComps.base + '.map\n'), 'utf8');
    } catch(e) {
        process.stderr.write('Unable to write to output path: ${outputFullPath}');
        process.exit(205);
    }
    
    if (options.sourceMaps === true)
    {
        try {
            fs.writeFileSync(outputPath + '.map', JSON.stringify(finalMap), 'utf8');
        } catch(e) {
            process.stderr.write('Unable to write source map to path: ${outputFullPath}.map');
            process.exit(206);
        }
    }
}
doIt();
